@@ -1,7 +1,7 @@
-type t = [ `RSA | `ED25519 | `P224 | `P256 | `P384 | `P521  ]
+type t = [ `RSA | `ED25519 | `P256 | `P384 | `P521  ]
 
 let strings =
-  [ ("rsa", `RSA) ; ("ed25519", `ED25519) ; ("p224", `P224) ;
+  [ ("rsa", `RSA) ; ("ed25519", `ED25519) ;
     ("p256", `P256) ; ("p384", `P384) ; ("p521", `P521) ]
 
 let to_string kt = fst (List.find (fun (_, k) -> kt = k) strings)
@@ -29,7 +29,7 @@ let supports_signature_scheme key_typ scheme =
   match key_typ, scheme with
   | `RSA, (`RSA_PSS | `RSA_PKCS1) -> true
   | `ED25519, `ED25519 -> true
-  | (`P224 | `P256 | `P384 | `P521), `ECDSA -> true
+  | (`P256 | `P384 | `P521), `ECDSA -> true
   | _ -> false
 
 let opt_signature_scheme ?scheme kt =
@@ -38,7 +38,7 @@ let opt_signature_scheme ?scheme kt =
   | None -> match kt with
     | `RSA -> `RSA_PSS
     | `ED25519 -> `ED25519
-    | `P224 | `P256 | `P384 | `P521 -> `ECDSA
+    | `P256 | `P384 | `P521 -> `ECDSA
 
 (* the default of RSA keys should be PSS, but most deployed certificates still
    use PKCS1 (and this library uses pkcs1 by default as well) *)
