@@ -12,5 +12,5 @@ let suites =
 
 let () =
   Printexc.record_backtrace true;
-  Mirage_crypto_rng_unix.initialize (module Mirage_crypto_rng.Fortuna);
+  Mirage_crypto_rng_unix.use_default ();
   Alcotest.run "X509 tests" suites
